library navigation_bar;

import 'dart:ui' as ui;

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'indicator.dart' as indicator;
import './i_tabbar.dart';

export 'indicator.dart' show IndicatorAxisAlignment;

class NavigationBar extends StatefulWidget {
  final List<String>? items;
  final double? labelWidth;
  final ItemBuilderDelegate? builderDelegate;
  final TabController? controller;
  final Color selectColor;
  final Color normalColor;
  final TextStyle? selectStyle;
  final TextStyle? normalStyle;
  final _Indicator? indicator;
  final bool isScrollable;
  final TabBarIndicatorSize indicatorSize;
  final EdgeInsetsGeometry? labelPadding;
  final Function(int? index)? onChange;
  final Function(int)? onTap;

  NavigationBar(
      {this.items,
      this.labelWidth,
      this.builderDelegate,
      this.controller,
      this.selectStyle,
      this.normalStyle,
      this.indicator,
      this.onChange,
      this.onTap,
      this.labelPadding,
      this.isScrollable = false,
      this.selectColor = Colors.black,
      this.normalColor = Colors.black12,
      this.indicatorSize = TabBarIndicatorSize.label,
      Key? key})
      : super(key: key) {
    assert(items != null || builderDelegate != null, '列表项不能为空');
  }

  @override
  State<StatefulWidget> createState() => NavigationBarState();
}

class NavigationBarState extends State<NavigationBar> {
  int? _index;
  TabController? _oldController;
  Future<ui.Image?> get indicatorImage async {
    if (isImageIndicator) {
      ImageIndicator indicator = widget.indicator as ImageIndicator;
      var image = await loadImage(indicator.image);
      return image;
    }

    return null;
  }

  Future<Decoration?> get indicatorDecoration async {
    if (isImageIndicator) {
      ImageIndicator imageIndicator = widget.indicator as ImageIndicator;
      return indicator.ImageIndicator(
          image: await indicatorImage,
          color: imageIndicator.color,
          width: imageIndicator.width,
          height: imageIndicator.height,
          alignment: imageIndicator.alignment,
          marginBottom: imageIndicator.marginBottom,
          marginRight: imageIndicator.marginRight);

    } else {
      RoundRectIndicator rectIndicator = widget.indicator as RoundRectIndicator;
      return indicator.RoundRectIndicator(
        color: rectIndicator.color,
        width: rectIndicator.width,
        height: rectIndicator.height,
        round: rectIndicator.round,
        marginBottom: rectIndicator.marginBottom,
        marginRight: rectIndicator.marginRight,
      );
    }
  }

  bool get isImageIndicator {
    return widget.indicator is ImageIndicator;
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    initEvnet();
  }

  @override
  void didUpdateWidget(covariant NavigationBar oldWidget) {
    super.didUpdateWidget(oldWidget);
    initEvnet();
  }

  @override
  void dispose() {
    if (_oldController != null) {
      _oldController!.removeListener(listenerCallback);
    }
    super.dispose();
  }

  void initEvnet() {
    if (_oldController != _getTabController()) {
      _oldController = _getTabController();
      _getTabController()!.removeListener(listenerCallback);
      _getTabController()!.addListener(listenerCallback);
    }
  }

  void listenerCallback() {
    if (_index == _getTabController()!.index) {
      return;
    }
    _index = _getTabController()!.index;
    if (widget.onChange != null) {
      widget.onChange!(_index);
    }
  }

  TabController? _getTabController() {
    if (widget.controller != null) {
      return widget.controller;
    } else {
      return DefaultTabController.of(context);
    }
  }

  @override
  Widget build(BuildContext context) {
    return FutureBuilder<Decoration?>(
        future: indicatorDecoration,
        builder: (BuildContext context, snapshot) {
          if (!snapshot.hasData) {
            return SizedBox();
          }
          List<Widget> tabs;
          if (widget.builderDelegate != null) {
            tabs = List.generate(widget.builderDelegate!.childCount, (index) {
              return widget.builderDelegate!.builder(context, index);
            });
          } else {
            tabs = widget.items!.map((text) {
              return Container(
                  width: widget.labelWidth,
                  height: double.infinity,
                  alignment: Alignment.center,
                  child: Text(
                    text,
                  ));
            }).toList();
          }
          return ITabBar(
            tabs: tabs,
            labelPadding: widget.labelPadding,
            controller: widget.controller,
            isScrollable: widget.isScrollable,
            onTap: (index) {
              if (widget.onTap != null) {
                widget.onTap!(index);
              }
            },
            labelStyle:
                TextStyle.lerp(TextStyle(height: 1), widget.selectStyle, 1),
            unselectedLabelStyle:
                TextStyle.lerp(TextStyle(height: 1), widget.normalStyle, 1),
            labelColor: widget.selectColor,
            unselectedLabelColor: widget.normalColor,
            indicatorSize: widget.indicatorSize,
            indicator: snapshot.data,
            indicatorColor: Colors.transparent,
          );
        });
  }

  Future<ui.Image> loadImage(String path) async {
    var data = await rootBundle.load(path);
    var codec = await ui.instantiateImageCodec(data.buffer.asUint8List());
    var info = await codec.getNextFrame();
    return info.image;
  }
}

class _Indicator {
  final Color? color;
  final double? width;
  final double? height;
  final double? marginBottom;
  final double? marginRight;

  _Indicator(
      {this.color,
      this.height,
      this.width,
      this.marginBottom = 0,
      this.marginRight = 0});
}

class ImageIndicator extends _Indicator {
  String image;
  indicator.IndicatorAxisAlignment alignment;

  ImageIndicator(
      {required this.image,
      Color? color,
      double? width,
      double? height,
      this.alignment = indicator.IndicatorAxisAlignment.center,
      double marginBottom = 0,
      double marginRight = 0})
      : super(
            color: color,
            width: width,
            height: height,
            marginBottom: marginBottom,
            marginRight: marginRight);
}

class RoundRectIndicator extends _Indicator {
  double round;
  RoundRectIndicator(
      {this.round = 1,
      required Color color,
      double? width,
      double height = 2,
      double? marginBottom,
      double? marginRight})
      : super(
            color: color,
            width: width,
            height: height,
            marginBottom: marginBottom,
            marginRight: marginRight);
}

class ItemBuilderDelegate {
  final int childCount;
  final Widget Function(BuildContext context, int index) builder;
  ItemBuilderDelegate({required this.childCount, required this.builder});
}
